#
# MakeHTML III R8
# Copyright (C) 1996-2001 Particle Corporation.
# Alex S. (Particle)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# - June 4th, 2001 - Alex S. (Particle)
#

use Cwd;

# if exactly 1 argument, then assume it's the only file...
if($#ARGV == 0){
   local $output = processfile($ARGV[0]);
   print $output;
   exit 0;
}

# any other case, accepting 2 directories
if($#ARGV < 1){
   print "usage: makehtml3.pl [indir] [outdir]\n";
   exit 1;
}

$src = $ARGV[0];
$dest = $ARGV[1];

# truncate copy file...
$currentWorkingDirectory = cwd;
if(-f "filecopy_dos.cmd"){
   truncate "filecopy_dos.cmd",0;
}

if(-f "filecopy_unix.sh"){
   truncate "filecopy_unix.sh",0;
}

# create destination (if doens't already exist)
if(! -d $dest){
   mkdir $dest,0777;
}

# work on src to dest and all subdirectories
chdir $src;
processor("$currentWorkingDirectory/$dest");
chdir $currentWorkingDirectory;


# function traverses through src dir and subdirs, for
# every file, if it's html, then calls the processor,
# else appends it to the copy file.
sub processor {
   local $destination = @_[0];
   my $var;
   for $var (<*>){
      if(-d $var){

         # directory, recursively do subdirs

         chdir $var;
         mkdir "$destination/$var",0777;
         processor("$destination/$var");
         chdir "..";
      }else{

         # it's a file...

         if($var =~ m/html$/ || $var =~ m/_ss.cgi$/ || $var =~ m/php$/){

            # it's an html file...

#            print "--PROCESSING FILE: $var to $destination/$var\n";

            # process and write out the output to the destination.
            local $output = processfile($var);
            $output =~ s/\x0D//gs;

            open(OUT,">$destination/$var") || die("cannot open $destination/$var for output");
            binmode OUT;
            print OUT $output;
            close OUT;


         }elsif($var !~ m/inl$/){

            # all other files (except inl files, which are used for script inclusions)

            open(OUT,">>$currentWorkingDirectory/filecopy_dos.cmd") || die("cannot open $currentWorkingDirectory/filecopy_dos.cmd for output");
            binmode OUT;

            open(OUT2,">>$currentWorkingDirectory/filecopy_unix.sh") || die("cannot open $currentWorkingDirectory/filecopy_unix.sh for output");
            binmode OUT2;

            $thecwd = cwd;
            if(length($thecwd) == 0){
               $thecwd = ".";
            }

            # make sure that the copy file contains OS specific copy commands.

            my $the_source = "$thecwd/$var";
            my $the_destination = "$destination/$var";
            $the_source =~ s/\//\\/gs;
            $the_destination =~ s/\//\\/gs;
            print OUT "copy $the_source $the_destination\n";
            close OUT;

            $the_source =~ s/^[a-zA-Z]://g;
            $the_destination =~ s/^[a-zA-Z]://g;
            $the_source =~ s/\\/\//gs;
            $the_destination =~ s/\\/\//gs;
            print OUT2 "cp $the_source $the_destination\n";
            close OUT2;


         }
      }
   }
}

# function opens file, switches directories, etc.,
# and calls a 'sub' process function repeatedly until
# the output doesn't have any scripts in it. It then
# returns the entire output as a string.
sub processfile {
   local $filename = @_[0];
   $filename =~ s/\\/\//g;
   local $dirname = $filename;
   local $theoutput;
   if($dirname =~ m/[\/]/){
      $dirname =~ s/[^\/]+$//;
      $filename =~ s/\Q$dirname\E//;
   }else{
      $dirname = "./";
   }
   local $cwd = cwd;
   chdir $dirname;
   if(-f $filename){
      open(IN,"<$filename") || die("file does not exist: $dirname$filename");
      local $entireFile = join('',<IN>);
      close(IN);
      $result = 1;
      $result_ref = \$result;
      $theoutput = $entireFile;

      # prepeatedly call the sub-process function until
      # there are no more scripts.

      if($filename =~ m/_ss.cgi$/){
         $theoutput = processfile2($theoutput,$result_ref,0);
      }else{
         while($result){
            $theoutput = processfile2($theoutput,$result_ref,1);
         }
      }

   }else{
      die("file does not exist: $dirname$filename");
   }
   chdir $cwd;
   return $theoutput;
}

# function parses the scripts/html from each other,
# evaluates the script, packages the output from
# script and html into a nice variable and returns it.
# if scripts are encountered, sets the result_ref reference
# to 1, else to 0.
sub processfile2 {
   local $filedata = @_[0];
   local $result_ref = @_[1];
   local $do_the_eval = @_[2];
   $_ = $filedata;
   local @tokens;
   ${$result_ref} = 0;

   # breakup input into script/non-script code.

   while(/^(.*?)<%(.*?)%>(.*)$/s){
      ${$result_ref} = 1;
      push @tokens,$1;
      push @tokens,"<%$2%>";
      $_ = $3;
   }
   push @tokens,$_;

   local @theoutputarray;
   push @theoutputarray,"local \@theoutput;\n";

   # go through all tokens...

   for $var (@tokens) {

      # if token is empty?
      if(length($var) == 0){

      }elsif($var =~ m/^<%(.*)%>$/s){

         # if token is a script...

         $var = $1;
         if($var =~ m/^=/s){

            # this is the variable print (similar to asp like code)

            $var =~ s/^=//s;
            push @theoutputarray,"push \@theoutput,\"$var\";\n";
         }elsif($var =~ m/^\@/s){

            # this is the command... similar to jsp code...

            $var =~ s/^\@\s*|\s*$//s;

            local @params = split(/\s+/,$var);
            $command = shift @params;

            local %arguments;
            for $tmp_val (@params) {
               my @valparam = split(/=/,$tmp_val);
               if($#valparam == 1){
                  my $valname = $valparam[0];
                  my $valvalue = $valparam[1];
                  $valvalue =~ s/^\"|\"$//gs;
                  $arguments{$valname} = $valvalue;
               }
            }

            local $commandResult = handleCommand($command,\%arguments);
            my @var_array = fourksplit($commandResult);
            my $tmp_var;
            for $tmp_var (@var_array) {
               push @theoutputarray,"push \@theoutput,'$tmp_var';\n";
            }
         }else{

            # this is pure Perl code (interpret as is)

            push @theoutputarray,"$var\n";
         }
      }else{

         # if token is plain html text

         my @var_array = fourksplit($var);
         my $tmp_var;
         for $tmp_var (@var_array) {
            push @theoutputarray,"push \@theoutput,'$tmp_var';\n";
         }
      }
   }

   # evaluate the output, and return it.


   if($do_the_eval){
      push @theoutputarray,"return join('',\@theoutput);\n";
      local $expr = join('',@theoutputarray);
      $theoutput = eval($expr);
   }else{
      push @theoutputarray,"print \"content-type: text/html\\n\\n\";\n";
      push @theoutputarray,"print \@theoutput;\n";
      unshift(@theoutputarray, "#!/usr/bin/perl\n");
      local $expr = join('',@theoutputarray);
      $theoutput = $expr;
   }
   return $theoutput;
}

# function to handle commands. first parameter
# is command, next parameter is a reference to
# a hash table with parameters.
sub handleCommand {
   local $command = @_[0];
   local $arguments_ref = @_[1];
   local $result;
   if($command eq "include"){

      # command is include

      $file = ${$arguments_ref}{"file"};
      $type = ${$arguments_ref}{"type"};
      if(length($file) > 0){

         if($type =~ m/code/){
            open(IN,"<$file");
            my @inc_lines = <IN>;
            close(IN);
            $inc_lines = join '',@inc_lines;
            $inc_lines =~ s/</&lt;/gs;
            $inc_lines =~ s/>/&gt;/gs;
            $inc_lines =~ s/\"/&quot;/gs;
            $result = $inc_lines;
         }elsif($type =~ m/copy/){

         }else{
            $result = processfile($file);

            # if name has parts of directories in it...
            if($file =~ m/\//){
               $file_directory = $file;
               $file_directory =~ s/[^\/]*$//;
               # redirect links on included files...
               $result = redirect_links($result,$file_directory);
            }

         }
      }else{
         die("command: $command requires a parameter: file");
      }
   }
   # return results of the command
   return $result;
}

# to through all the anchors, and img tags, and redirect the location for
# includED files to the corrected path of the includING file.
sub redirect_links {
   my ($input,$dir) = @_;
   $_ = $input;
   my @t;
   my $url;
   while(/^(.*?)<(.*?)>(.*)$/s){
      push @t,$1;
      $_ = $3;
      $tag = $2;
      if($tag =~ m/[Aa].*?[Hh][Rr][Ee][Ff]="([^\"]*?)"/s ||
         $tag =~ m/[Ii][Mm][Gg].*?[Ss][Rr][Cc]="([^\"]*?)"/s){
         $url = $1;
         if($url !~ m/^http:/s && $url !~ m/^ftp:/s && $url !~ m/^mailto:/s){
            $tag =~ s/\Q$url\E/$dir$url/s;
         }
      }
      push @t,"<$tag>";
   }
   push @t,$_;
   return join '',@t;
}

# function to split the input into 4k or less chunks.
# returns an array of these chunks
sub fourksplit {
   my $thevar = @_[0];
   local @thearray;
   my $len = length($thevar);
   my $i;
   for($i=0;$i<$len;$i+=4000){
      my $j = $i + 4000;
      my $value;
      if($j < $len){
         $value = substr($thevar,$i,4000);
      }else{
         $value = substr($thevar,$i,$len - $i);
      }
      $value =~ s/\\/\\\\/sg;
      $value =~ s/\'/\\\'/sg;
      push @thearray,$value;
   }
   return @thearray;
}



